#!/bin/bash
#
# This script generates the cpio archive used as input to initramdisk.
# Should be called within a fakeroot environment to allow root file creation
# as we generate cpio archive from a directory tree.
# File list format is the same as the one used by linux kernel gen_init_cpio.c
#

# Usage:
#  ./gencpio.sh [ flasher ]
# 
# TODO Set back this to ${MACH} and use relative path in initramfs_list
# srcdir_prefix=${MACH}/
srcdir_prefix=.
tmproot=./tmproot
ext="$1"
if [ -n "${ext}" ]; then
	ext="-${ext}"
fi

dir() {
	dirname=${1:1}
	perms=$2
	mkdir -p -m 0"$perms" "$tmproot/$dirname" || exit 1
}

file() {
	target=${1:1}
	source=$2
	perms=$3
	if [ "${source::1}" != "/" ]; then
		cp -p "$srcdir_prefix/$source" "$tmproot/$target" || exit 1
	else
		cp -p "$source" "$tmproot/$target" || exit 1
	fi
	chmod 0"$perms" "$tmproot/$target" || exit 1
}

nod() {
	nodname=${1:1}
	perms=$2
	nodtype=$5
	major=$6
	minor=$7
	mknod -m 0"$perms" "$tmproot/$nodname" $nodtype $major $minor || exit 1
}

slink() {
	target=${1:1}
	source=$2
	perms=$3
	ln -s "$source" "$tmproot/$target" || exit 1
}

rm -Rf "$tmproot"
mkdir "$tmproot"
if [ ! -f ./${MACH}${ext}/initramfs_list ]
then
	echo initramfs_list missing in directory ${MACH}${ext}
	exit 1
fi
. ./${MACH}${ext}/initramfs_list
(	cd $tmproot
	find . | cpio -o -Hnewc | gzip > ../initramfs.cpio.gz.new
	find . -type f -exec md5sum \{\} \; | sort > ../initramfs.md5.new
)
rm -Rf "$tmproot"

# Note: We need to hack to check if content has been modified because we
# store date/time in .cpio of newly created nodes/symlinks, so md5sum of
# the generated cpio is always different, which force a new kernel build.
# To prevent this we store a md5sum of all files/nodes/symlinks, check if
# there is a md5 sum stored, and if so, if all checksum matches.
if [ -f initramfs.cpio.gz -a -f initramfs.md5 ]; then
	if diff initramfs.md5 initramfs.md5.new > /dev/null; then
		echo "No content modification, keeping old .cpio.gz"
		rm initramfs.cpio.gz.new initramfs.md5.new
	else
		echo "Content modified, using new .cpio.gz"
		rm initramfs.cpio.gz initramfs.md5
		mv initramfs.md5.new initramfs.md5
		mv initramfs.cpio.gz.new initramfs.cpio.gz
	fi
else
	echo "Using new .cpio.gz"
	rm -f initramfs.md5 initramfs.cpio.gz
	mv initramfs.md5.new initramfs.md5
	mv initramfs.cpio.gz.new initramfs.cpio.gz
fi
